@@ -1,4 +1,6 @@
+import re
 import subprocess
+import tempfile
 import unittest
 from fnmatch import fnmatch
 import multiprocessing
@@ -30,6 +32,8 @@ try:
     multiprocessing.set_start_method('fork')
 except ValueError:  # noqa, If platform does not support, just ignore
     pass
+except RuntimeError:  # noqa, If context has already being set, just ignore
+    pass
 
 
 def get_stdout_logger():
@@ -93,24 +97,7 @@ def get_file_name_extension(path):
     return Path(path).suffix[1:]
 
 
-def get_normalized_path(dir_):
-    """
-
-    >>> get_normalized_path('abc/xyz/')
-    'abc/xyz'
-    >>> get_normalized_path('abc/xyz')
-    'abc/xyz'
-    >>> get_normalized_path('abc/xyz.txt')
-    'abc/xyz.txt'
-
-    """
-    if dir_[-1] == '/':
-        return dir_[:-1]
-    else:
-        return dir_
-
-
-def get_dir_without_last_slash(path):
+def get_dir_without_last_slash(path: str) -> str:
     """
 
     >>> get_dir_without_last_slash('abc/src01_hrc01.yuv')
@@ -123,7 +110,7 @@ def get_dir_without_last_slash(path):
     'abc/xyz'
 
     """
-    return "/".join(path.split("/")[:-1])
+    return os.path.dirname(path)
 
 
 def make_parent_dirs_if_nonexist(path):
@@ -197,31 +184,61 @@ def indices(a, func):
     return [i for (i, val) in enumerate(a) if func(val)]
 
 
-def import_python_file(filepath):
+def import_python_file(filepath : str, override : dict = None):
     """
-    Import a python file as a module.
-    :param filepath:
-    :return:
+    Import a python file as a module, allowing overriding some of the variables.
+    Assumption: in the original python file, variables to be overridden get assigned once only, in a single line.
     """
-    filename = get_file_name_without_extension(filepath)
-    try:
-        from importlib.machinery import SourceFileLoader
-        ret = SourceFileLoader(filename, filepath).load_module()
-    except ImportError:
-        import imp
-        ret = imp.load_source(filename, filepath)
-    return ret
+    if override is None:
+        filename = get_file_name_without_extension(filepath)
+        try:
+            from importlib.machinery import SourceFileLoader
+            ret = SourceFileLoader(filename, filepath).load_module()
+        except ImportError:
+            import imp
+            ret = imp.load_source(filename, filepath)
+        return ret
+    else:
+        override_ = override.copy()
+        tmpfile = tempfile.NamedTemporaryFile(delete=False, suffix='.py')
+        with open(filepath, 'r') as fin:
+            with open(tmpfile.name, 'w') as fout:
+                while True:
+                    line = fin.readline()
+                    if len(override_) > 0:
+                        suffixes = []
+                        for key in list(override_.keys()):
+                            if key in line and '=' in line:
+                                s = f"{key} = '{override_[key]}'" if isinstance(override_[key], str) else f"{key} = {override_[key]}"
+                                suffixes.append(s)
+                                del override_[key]
+                        if len(suffixes) > 0:
+                            line = '\n'.join([line.strip()] + suffixes) + '\n'
+                    fout.write(line)
+                    if not line:
+                        break
+                if len(override_) > 0:
+                    for key in override_:
+                        s = f"{key} = '{override_[key]}'" if isinstance(override_[key], str) else f"{key} = {override_[key]}"
+                        s += '\n'
+                        fout.write(s)
+        #============= debug =================
+        # with open(tmpfile.name, 'r') as fin:
+        #     print(fin.read())
+        #=====================================
+        ret = import_python_file(tmpfile.name)
+        os.remove(tmpfile.name)
+        return ret
 
 
-def make_absolute_path(path, current_dir):
-    '''
-
+def make_absolute_path(path: str, current_dir: str) -> str:
+    """
     >>> make_absolute_path('abc/cde.fg', '/xyz/')
     '/xyz/abc/cde.fg'
     >>> make_absolute_path('/abc/cde.fg', '/xyz/')
     '/abc/cde.fg'
-
-    '''
+    """
+    assert current_dir.endswith('/'), f"expect current_dir ends with '/', but is: {current_dir}"
     if path[0] == '/':
         return path
     else:
@@ -513,7 +530,7 @@ class MyTestCase(unittest.TestCase):
         self.maxDiff = None
 
     def tearDown(self):
-        self.assertEqual([], self.verificationErrors)
+        unittest.TestCase.assertEqual(self, [], self.verificationErrors)
 
     def assertAlmostEqual(self, first, second, places=None, msg=None, delta=None):
         try:
@@ -521,11 +538,24 @@ class MyTestCase(unittest.TestCase):
         except AssertionError as e:
             self.verificationErrors.append(str(e))
 
+    def assertEqual(self, first, second, msg=None):
+        try:
+            super().assertEqual(first, second, msg)
+        except AssertionError as e:
+            self.verificationErrors.append(str(e))
+
+    def assertTrue(self, expr, msg=None):
+        try:
+            super().assertTrue(expr, msg)
+        except AssertionError as e:
+            self.verificationErrors.append(str(e))
+
 
 class QualityRunnerTestMixin(object):
 
-    def run_each(self, score, runner_class, asset, optional_dict,
-                 optional_dict2=None, result_store=None, places=5):
+    @staticmethod
+    def _run_each_no_assert(runner_class, asset, optional_dict,
+                            optional_dict2=None, result_store=None, **more):
         runner = runner_class(
             [asset],
             None,
@@ -536,10 +566,29 @@ class QualityRunnerTestMixin(object):
             optional_dict2=optional_dict2,
         )
         runner.run(parallelize=False)
-        results = runner.results
-        self.assertAlmostEqual(results[0][runner_class.get_score_key()],
+        result = runner.results[0]
+        return result
+
+    def run_each(self, score, runner_class, asset, optional_dict,
+                 optional_dict2=None, result_store=None, places=5, **more):
+        result = self._run_each_no_assert(
+            runner_class, asset, optional_dict,
+            optional_dict2, result_store, **more)
+        self.assertAlmostEqual(result[runner_class.get_score_key()],
                                score, places=places)
 
+    def plot_frame_scores(self, ax, *args, **kwargs):
+        result = self._run_each_no_assert(*args, **kwargs)
+        runner_class, asset, optional_dict = args
+        avg_score = result[runner_class.get_score_key()]
+        label = [f'avg. {runner_class.TYPE}: {avg_score:.3f}']
+        if 'label' in kwargs:
+            label += [kwargs['label']]
+        label = ', '.join(label)
+        ax.set_xlabel('Frame Number')
+        ax.set_ylabel('Score')
+        ax.plot(result[runner_class.get_scores_key()], label=label)
+
 
 def find_linear_function_parameters(p1, p2):
     """
@@ -713,6 +762,79 @@ def round_up_to_odd(f):
     """
     return int(np.ceil(f) // 2 * 2 + 1)
 
+
+class NoPrint(object):
+
+    def __enter__(self):
+        self._original_stdout = sys.stdout
+        sys.stdout = open(os.devnull, 'w')
+
+    def __exit__(self, exc_type, exc_val, exc_tb):
+        sys.stdout.close()
+        sys.stdout = self._original_stdout
+
+
+def linear_func(x, a, b):
+    return a*x + b
+
+
+def linear_fit(x, y):
+    """
+    >>> fit = linear_fit([0, 1], [0, 1])
+    >>> (fit[0][0], fit[0][1])
+    (1.0, 0.0)
+    """
+    assert isinstance(x, (list, tuple, np.ndarray)), 'x must be a list, tuple, or a numpy array'
+    assert len(x) == np.size(x) and len(x) > 0, 'x must be one-dimensional with non-zero length'
+    assert isinstance(y, (list, tuple, np.ndarray)), 'y must be a list or a numpy array'
+    assert len(y) == np.size(y) and len(y) > 0, 'y must be one-dimensional with non-zero length'
+    assert len(x) == len(y), 'x must be the same length as y'
+
+    import scipy.optimize
+    return scipy.optimize.curve_fit(linear_func, x, y, [1.0, 0.0])
+
+
+def map_yuv_type_to_bitdepth(yuv_type):
+    """
+    >>> map_yuv_type_to_bitdepth('yuv420p')
+    8
+    >>> map_yuv_type_to_bitdepth('yuv422p')
+    8
+    >>> map_yuv_type_to_bitdepth('yuv444p')
+    8
+    >>> map_yuv_type_to_bitdepth('yuv420p10le')
+    10
+    >>> map_yuv_type_to_bitdepth('yuv422p10le')
+    10
+    >>> map_yuv_type_to_bitdepth('yuv444p10le')
+    10
+    >>> map_yuv_type_to_bitdepth('yuv420p12le')
+    12
+    >>> map_yuv_type_to_bitdepth('yuv422p12le')
+    12
+    >>> map_yuv_type_to_bitdepth('yuv444p12le')
+    12
+    >>> map_yuv_type_to_bitdepth('yuv420p16le')
+    16
+    >>> map_yuv_type_to_bitdepth('yuv422p16le')
+    16
+    >>> map_yuv_type_to_bitdepth('yuv444p16le')
+    16
+    >>> map_yuv_type_to_bitdepth('notyuv') is None
+    True
+    """
+    if yuv_type in ['yuv420p', 'yuv422p', 'yuv444p']:
+        return 8
+    elif yuv_type in ['yuv420p10le', 'yuv422p10le', 'yuv444p10le']:
+        return 10
+    elif yuv_type in ['yuv420p12le', 'yuv422p12le', 'yuv444p12le']:
+        return 12
+    elif yuv_type in ['yuv420p16le', 'yuv422p16le', 'yuv444p16le']:
+        return 16
+    else:
+        return None
+
+
 if __name__ == '__main__':
     import doctest
     doctest.testmod()
